#ifndef _C_INTEGRAL_TO_DISTANCE_H
#define _C_INTEGRAL_TO_DISTANCE_H

float _c_integral_to_distance(
    const float* integral,
    int rows,
    int cols,
    int row,
    int col,
    int offset,
    float h2s2);

#endif  // _C_INTEGRAL_TO_DISTANCE_H
